# -*- coding: utf-8 -*-
"""
Created on Sun Nov 11 23:57:03 2018

@author: shane

This file is part of nutra, a nutrient analysis program.
    https://github.com/nutratech/cli
    https://pypi.org/project/nutra/

nutra is an extensible nutrient analysis and composition application.
Copyright (C) 2018-2020  Shane Jaroch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import csv
import shutil

from colorama import Fore, Style
from tabulate import tabulate

from ..utils import (
    COLOR_CRIT,
    COLOR_DEFAULT,
    COLOR_OVER,
    COLOR_WARN,
    NUTR_ID_CARBS,
    NUTR_ID_FAT_TOT,
    NUTR_ID_FIBER,
    NUTR_ID_KCAL,
    NUTR_ID_PROTEIN,
    THRESH_CRIT,
    THRESH_OVER,
    THRESH_WARN,
)
from ..persistence import TESTING, VERBOSITY
from ..persistence.sql.usda.funcs import (
    analyze_foods,
    food_details,
    nutrients_overview,
    servings,
)


def foods_analyze(food_ids, grams=None):
    """Analyze a list of food_ids against stock RDA values"""

    # Get analysis
    analysis = analyze_foods(food_ids)
    analyses = {}
    for a in analysis:
        id = a[0]
        if grams is not None:
            anl = (a[1], round(a[2] * grams / 100, 2))
        else:
            anl = (a[1], a[2])
        if id not in analyses:
            analyses[id] = [anl]
        else:
            analyses[id].append(anl)
    # serving = servings()[1]
    serving = servings(food_ids)
    food_des = food_details(food_ids)
    food_des = {x[0]: x for x in food_des}
    nutrients = nutrients_overview()
    rdas = {x[0]: x[1] for x in nutrients.values()}

    # --------------------------------------
    # Food-by-food analysis (w/ servings)
    # --------------------------------------
    servings_tables = []
    nutrients_tables = []
    for food_id in analyses:
        food_name = food_des[food_id][2]
        # food_name = food["long_desc"]
        print(
            "\n======================================\n"
            f"==> {food_name} ({food_id})\n"
            "======================================\n",
        )
        print("\n=========================\nSERVINGS\n=========================\n")

        ###############
        # Serving table
        headers = ["msre_id", "msre_desc", "grams"]
        # Copy obj with dict(x)
        rows = [(x[1], x[2], x[3]) for x in serving if x[0] == food_id]
        # for r in rows:
        #     r.pop("food_id")
        # Print table
        servings_table = tabulate(rows, headers=headers, tablefmt="presto")
        print(servings_table)
        servings_tables.append(servings_table)

        refuse = next(
            ((x[7], x[8]) for x in food_des.values() if x[0] == food_id and x[7]), None
        )
        if refuse:
            print("\n=========================\nREFUSE\n=========================\n")
            print(refuse[0])
            print(f"    ({refuse[1]}%, by mass)")

        print("\n=========================\nNUTRITION\n=========================\n")

        ################
        # Nutrient table
        headers = ["id", "nutrient", "rda", "amount", "units"]
        rows = []
        # food_nutes = {x["nutr_id"]: x for x in food["nutrients"]}
        # for id, nute in food_nutes.items():
        for id, amount in analyses[food_id]:
            # Skip zero values
            # amount = food_nutes[id]["nutr_val"]
            if not amount:
                continue

            nutr_desc = nutrients[id][4] if nutrients[id][4] else nutrients[id][3]
            unit = nutrients[id][2]

            # Insert RDA % into row
            if rdas[id]:
                rda_perc = str(round(amount / rdas[id] * 100, 1)) + "%"
            else:
                # print(rdas[id])
                rda_perc = None
            row = [id, nutr_desc, rda_perc, round(amount, 2), unit]

            rows.append(row)

        # Print table
        table = tabulate(rows, headers=headers, tablefmt="presto")
        print(table)
        nutrients_tables.append(table)

    return nutrients_tables, servings_tables


def day_analyze(day_csv_paths, rda_csv_path=None):
    """Analyze a day optionally with custom RDAs,
    e.g.  nutra day ~/.nutra/rocky.csv -r ~/.nutra/dog-rdas-18lbs.csv
    TODO: Should be a subset of foods_analyze
    """
    rda = []
    if rda_csv_path:
        fp = open(rda_csv_path)
        rda_csv_input = csv.DictReader(row for row in fp if not row.startswith("#"))
        rda = list(rda_csv_input)

    logs = []
    food_ids = set()
    for day_csv_path in day_csv_paths:
        fp = open(day_csv_path)
        day_csv_input = csv.DictReader(row for row in fp if not row.startswith("#"))
        log = list(day_csv_input)
        for entry in log:
            try:
                food_ids.add(int(entry["id"]))
            except Exception as e:
                if TESTING or VERBOSITY > 1:
                    print(repr(e))
        logs.append(log)

    # Inject user RDAs
    nutrients = [list(x) for x in nutrients_overview().values()]
    for r in rda:
        id = int(r["id"])
        _rda = float(r["rda"])
        for n in nutrients:
            if n[0] == id:
                n[1] = _rda
                if VERBOSITY > 1:
                    print(f"INJECT RDA: {f'{_rda} {n[2]}'.ljust(12)} -->  {n[4]}")
    nutrients = {x[0]: x for x in nutrients}

    # Analyze foods
    _foods_analysis = analyze_foods(food_ids)
    foods_analysis = {}
    for f in _foods_analysis:
        id = f[0]
        anl = f[1], f[2]
        if id not in foods_analysis:
            foods_analysis[id] = [anl]
        else:
            foods_analysis[id].append(anl)

    # Compute totals
    nutrients_totals = []
    for log in logs:
        nutrient_totals = {}
        for entry in log:
            if entry["id"]:
                id = int(entry["id"])
                grams = float(entry["grams"])
                for n in foods_analysis[id]:
                    nutr_id = n[0]
                    nutr_per_100g = n[1]
                    nutr_val = grams / 100 * nutr_per_100g
                    if nutr_id not in nutrient_totals:
                        nutrient_totals[nutr_id] = nutr_val
                    else:
                        nutrient_totals[nutr_id] += nutr_val
        nutrients_totals.append(nutrient_totals)

    #######
    # Print
    w = shutil.get_terminal_size()[0]
    buffer = w - 4 if w > 4 else w
    for analysis in nutrients_totals:
        day_format(analysis, nutrients, buffer=buffer)
    return nutrients_totals


def day_format(analysis, nutrients, buffer=None):
    def print_header(header):
        print(Fore.CYAN, end="")
        print("~~~~~~~~~~~~~~~~~~~~~~~~~~~")
        print(f"→ {header}")
        print("~~~~~~~~~~~~~~~~~~~~~~~~~~~")
        print(Style.RESET_ALL)

    def print_macro_bar(fat, net_carb, pro, kcals_max, buffer=None):
        kcals = fat * 9 + net_carb * 4 + pro * 4

        p_fat = (fat * 9) / kcals
        p_carb = (net_carb * 4) / kcals
        p_pro = (pro * 4) / kcals

        # TODO: handle rounding cases, tack on to, or trim off FROM LONGEST ?
        mult = kcals / kcals_max
        n_fat = round(p_fat * buffer * mult)
        n_carb = round(p_carb * buffer * mult)
        n_pro = round(p_pro * buffer * mult)

        # Headers
        f_buf = " " * (n_fat // 2) + "Fat" + " " * (n_fat - n_fat // 2 - 3)
        c_buf = " " * (n_carb // 2) + "Carbs" + " " * (n_carb - n_carb // 2 - 5)
        p_buf = " " * (n_pro // 2) + "Pro" + " " * (n_pro - n_pro // 2 - 3)
        print(
            f"  {Fore.YELLOW}{f_buf}{Fore.BLUE}{c_buf}{Fore.RED}{p_buf}{Style.RESET_ALL}"
        )

        # Bars
        print(" <", end="")
        print(Fore.YELLOW + "=" * n_fat, end="")
        print(Fore.BLUE + "=" * n_carb, end="")
        print(Fore.RED + "=" * n_pro, end="")
        print(Style.RESET_ALL + ">")

        # Calorie footers
        k_fat = str(round(fat * 9))
        k_carb = str(round(net_carb * 4))
        k_pro = str(round(pro * 4))
        f_buf = " " * (n_fat // 2) + k_fat + " " * (n_fat - n_fat // 2 - len(k_fat))
        c_buf = (
            " " * (n_carb // 2) + k_carb + " " * (n_carb - n_carb // 2 - len(k_carb))
        )
        p_buf = " " * (n_pro // 2) + k_pro + " " * (n_pro - n_pro // 2 - len(k_pro))
        print(
            f"  {Fore.YELLOW}{f_buf}{Fore.BLUE}{c_buf}{Fore.RED}{p_buf}{Style.RESET_ALL}"
        )

    def print_nute_bar(n_id, amount, nutrients):
        nutrient = nutrients[n_id]
        rda = nutrient[1]
        tag = nutrient[3]
        unit = nutrient[2]
        # anti = nutrient[5]

        if not rda:
            return False, nutrient
        attain = amount / rda
        perc = round(100 * attain, 1)

        if attain >= THRESH_OVER:
            color = COLOR_OVER
        elif attain <= THRESH_CRIT:
            color = COLOR_CRIT
        elif attain <= THRESH_WARN:
            color = COLOR_WARN
        else:
            color = COLOR_DEFAULT

        # Print
        detail_amount = f"{round(amount, 1)}/{rda} {unit}".ljust(18)
        detail_amount = f"{detail_amount} -- {tag}"
        li = 20
        l = round(li * attain) if attain < 1 else li
        print(f" {color}<", end="")
        print("=" * l + " " * (li - l) + ">", end="")
        print(f" {perc}%\t[{detail_amount}]", end="")
        print(Style.RESET_ALL)

        return (True,)

    # Actual values
    kcals = round(analysis[NUTR_ID_KCAL])
    pro = analysis[NUTR_ID_PROTEIN]
    net_carb = analysis[NUTR_ID_CARBS] - analysis[NUTR_ID_FIBER]
    fat = analysis[NUTR_ID_FAT_TOT]
    kcals_449 = round(4 * pro + 4 * net_carb + 9 * fat)

    # Desired values
    kcals_rda = round(nutrients[NUTR_ID_KCAL][1])
    pro_rda = nutrients[NUTR_ID_PROTEIN][1]
    net_carb_rda = nutrients[NUTR_ID_CARBS][1] - nutrients[NUTR_ID_FIBER][1]
    fat_rda = nutrients[NUTR_ID_FAT_TOT][1]

    # Print calories and macronutrient bars
    print_header("Macronutrients")
    kcals_max = max(kcals, kcals_rda)
    print(
        f"Actual:    {kcals} kcal ({round(kcals * 100 / kcals_rda, 1)}% RDA), {kcals_449} by 4-4-9"
    )
    print_macro_bar(fat, net_carb, pro, kcals_max, buffer=buffer)
    print(f"\nDesired:   {kcals_rda} kcal ({'%+d' % (kcals - kcals_rda)} kcal)")
    print_macro_bar(
        fat_rda,
        net_carb_rda,
        pro_rda,
        kcals_max,
        buffer=buffer,
    )

    # Nutrition detail report
    print_header("Nutrition detail report")
    for n_id in analysis:
        print_nute_bar(n_id, analysis[n_id], nutrients)
    # TODO: below
    print(
        "work in progress...some minor fields with negligible data, they are not shown here"
    )



from . import con, _sql
from .. import profile_id


def nt_ver():
    """Gets version string for nt.sqlite database"""
    if con is None:
        return None
    query = "SELECT * FROM version;"
    result = _sql(query)
    return result[-1][1]


# ----------------------
# Recipe functions
# ----------------------


def recipe_add():
    query = """
"""
    return _sql(query)


def recipes():
    query = """
SELECT
  id,
  name,
  COUNT(recipe_id) AS n_foods,
  SUM(grams) AS grams,
  guid,
  created
FROM
  recipes
  LEFT JOIN recipe_dat ON recipe_id = id
GROUP BY
  id;
"""
    return _sql(query, headers=True)


def analyze_recipe(id):
    query = f"""
SELECT
  id,
  name,
  food_id,
  grams
FROM
  recipes
  INNER JOIN recipe_dat ON recipe_id = id
    AND id = {id};
"""
    return _sql(query)


def recipe(id):
    query = "SELECT * FROM recipes WHERE id=?;"
    return _sql(query, (id,))


# ----------------------------
# Profile and Sync functions
# ----------------------------


def sql_profile_guid_from_id(profile_id):
    query = f"SELECT guid FROM profiles WHERE id={profile_id}"
    return _sql(query)[0][0]


def sql_last_sync():
    query = """
SELECT
  max(
    (SELECT IFNULL(max(last_sync), -1) FROM profiles),
    (SELECT IFNULL(max(last_sync), -1) FROM biometric_log),
    (SELECT IFNULL(max(last_sync), -1) FROM recipes),
    (SELECT IFNULL(max(last_sync), -1) FROM food_log),
    (SELECT IFNULL(max(last_sync), -1) FROM recipe_log),
    (SELECT IFNULL(max(last_sync), -1) FROM rda)
  ) AS last_sync;
"""
    return _sql(query)[0][0]


def sql_inserted_or_updated_entities(last_sync):
    query = f"SELECT * FROM profiles WHERE updated>{last_sync}"
    profiles = _sql(query)
    query = f"""
SELECT
  *
FROM
  biometric_log
  INNER JOIN bio_log_entry entry ON biometric_log.id = entry.log_id
WHERE updated>{last_sync};
"""
    # SELECT * FROM biometric_log WHERE updated>{last_sync}"
    bio_logs = _sql(query)

    return profiles, bio_logs


# ----------------------
# Biometric functions
# ----------------------


def sql_biometrics():
    query = "SELECT * FROM biometrics;"
    return _sql(query, headers=True)


def sql_biometric_logs(profile_id):
    query = "SELECT * FROM biometric_log WHERE profile_id=?"
    return _sql(query, args=(profile_id,), headers=True)


def sql_biometric_add(bio_vals):
    cur = con.cursor()

    # TODO: finish up
    query1 = "INSERT INTO biometric_log(profile_id, tags, notes) VALUES (?, ?, ?)"
    result = _sql(query1, (profile_id, "", ""))
    id = cur.lastrowid
    print(id)



import os
import sys

from dotenv import load_dotenv


# Check Python version
PY_MIN_VER = (3, 6, 5)
PY_MIN_STR = ".".join(str(x) for x in PY_MIN_VER)
if sys.version_info < PY_MIN_VER:
    ver = ".".join([str(x) for x in sys.version_info[0:3]])
    print("ERROR: nutra requires Python %s or later to run" % PY_MIN_STR)
    print("HINT:  You're running Python " + ver)
    exit(1)

# Read in .env file if it exists locally, else look to env vars
load_dotenv(verbose=False)

ROOT_DIR = os.path.abspath(os.path.dirname(__file__))
NUTRA_DIR = os.path.join(os.path.expanduser("~"), ".nutra")
USDA_DB_NAME = "usda.sqlite"
NT_DB_NAME = "nt.sqlite"

# Set DB versions here
__db_target_usda__ = "0.0.8"
__db_target_nt__ = "0.0.0"


# Package info
__title__ = "nutra"
__version__ = "0.2.0.dev13"
__author__ = "Shane Jaroch"
__license__ = "GPL v3"
__copyright__ = "Copyright 2018-2020 Shane Jaroch"



from datetime import datetime

import requests


class Response:
    def __init__(self, url, auth_token=None):
        self.url = url
        self.initial_time = datetime.now()
        self.auth_token = auth_token
        self.headers = {"Authorization": f"Bearer {auth_token}"} if auth_token else None

    def build_response(self):
        self.code = self.Response.status_code  # or self.res["code"]

        # TODO: try/catch if server returns XML/HTML error, .json() will fail..
        self.res = self.Response.json()
        self.data = self.res["data"]
        self.ok = self.res["status"] == "OK"
        if not self.ok:
            self.error = self.data["error"]

        self.program = self.res["program"]
        self.version = self.res["version"]
        self.release = self.res["release"]

        self.server_datetime = datetime.fromtimestamp(self.res["timestamp"])
        self.final_time = datetime.now()
        self.delay = round((self.final_time - self.initial_time).total_seconds() * 1000)

    #
    # HTTP Methods
    def get(self, params=None):
        self.params = params
        self.Response = requests.get(self.url, params=params, headers=self.headers)
        self.build_response()

    def delete(self):
        self.Response = requests.delete(self.url, headers=self.headers)
        self.build_response()

    def post(self, body=None):
        self.body = body
        self.Response = requests.post(self.url, json=body, headers=self.headers)
        self.build_response()

    def patch(self, body=None):
        self.body = body
        self.Response = requests.patch(self.url, json=body, headers=self.headers)
        self.build_response()



import json

import requests

from .. import NUTRA_DIR
from ..persistence.sql import profile_guid
from ..persistence.sql.nt import sql_inserted_or_updated_entities, sql_last_sync
from ..persistence import SERVER_HOST, login_token


def sync():
    def get():
        params = {"uid": profile_guid, "last_sync": last_sync}

        print(f"GET {url}")
        response = requests.get(
            url, params=params, headers={"Authorization": f"Bearer {login_token}"}
        )
        res = response.json()
        data = res["data"]
        if "error" in data:
            print("error: " + data["error"])
            return

        print(data)

    def post():
        profiles, bio_logs = sql_inserted_or_updated_entities(last_sync)
        data = {
            "uid": profile_guid,
            "entities": {"profiles": profiles, "bio_logs": bio_logs},
        }

        print(f"POST {url}")
        response = requests.post(
            url, json=data, headers={"Authorization": f"Bearer {login_token}"}
        )
        res = response.json()
        data = res["data"]
        if "error" in data:
            print("error: " + data["error"])
            return

        print(data)

    # Make GET and POST reqeusts to /sync
    url = f"{SERVER_HOST}/sync"
    last_sync = sql_last_sync()
    get()
    post()


def register(email, password):
    print("not implemented ;]")


def login(email, password):
    import getpass
    import socket
    import sys

    hostname = socket.gethostname()
    username = getpass.getuser()
    oper_sys = sys.platform

    url = f"{SERVER_HOST}/v2/login"
    print(f"POST {url}")
    response = requests.post(
        url,
        json={
            "email": email,
            "password": password,
            "os": oper_sys,
            "hostname": hostname,
            "username": username,
        },
    )
    res = response.json()
    data = res["data"]
    if "error" in data:
        print("error: " + data["error"])
        return

    with open(f"{NUTRA_DIR}/prefs.json", "r") as f:
        prefs_json = json.load(f)

    with open(f"{NUTRA_DIR}/prefs.json", "w+") as f:
        prefs_json["email"] = email
        prefs_json["token"] = data["token"]
        f.write(json.dumps(prefs_json, indent=4))
    print("Logged in.")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 12 15:14:00 2020

@author: shane
"""

from tabulate import tabulate

from ..persistence.sql.nt.funcs import (
    analyze_recipe,
    recipes as _recipes,
    recipe as _recipe_overview,
)
from ..persistence.sql.usda.funcs import analyze_foods, nutrients_overview, food_details
from ..utils.nutprogbar import print_nutprogbar


def recipes_overview():
    recipes = _recipes()[1]

    results = []
    for recipe in recipes:
        result = {
            "id": recipe[0],
            "name": recipe[1],
            "n_foods": recipe[2],
            "weight": recipe[3],
            # "guid": recipe[4],
            # "created": recipe[5],
        }
        results.append(result)

    table = tabulate(results, headers="keys", tablefmt="presto")
    print(table)
    return results


def recipe_overview(id):

    recipe = analyze_recipe(id)

    try:
        name = recipe[0][1]
    except Exception as e:
        print(repr(e))
        return None

    print(name)

    food_ids = {x[2]: x[3] for x in recipe}
    food_names = {x[0]: x[3] for x in food_details(food_ids.keys())}
    food_analyses = analyze_foods(food_ids.keys())

    table = tabulate(
        [[food_names[id], grams] for id, grams in food_ids.items()],
        headers=["food", "g"],
    )
    print(table)
    # tabulate nutrient RDA %s
    nutrients = nutrients_overview()
    # rdas = {x[0]: x[1] for x in nutrients.values()}
    print_nutprogbar(food_ids, food_analyses, nutrients)

    return recipe


def recipe_add(name, food_amts):
    print()
    print("New recipe: " + name + "\n")

    food_names = {x[0]: x[2] for x in food_details(food_amts.keys())}

    results = []
    for id, grams in food_amts.items():
        results.append([id, food_names[id], grams])

    table = tabulate(results, headers=["id", "food_name", "grams"], tablefmt="presto")
    print(table)

    confirm = input("\nCreate recipe? [Y/n] ")

    if confirm.lower() == "y":
        print("not implemented ;]")


def recipe_edit(id):

    recipe = _recipe_overview(id)

    print(recipe[1])
    confirm = input("Do you wish to edit? [Y/n] ")

    if confirm.lower() == "y":
        print("not implemented ;]")



from tabulate import tabulate

from ..persistence import profile_id
from ..persistence.sql.nt.funcs import (
    con,
    sql_biometric_add,
    sql_biometric_logs,
    sql_biometrics,
)


def biometrics():
    headers, rows = sql_biometrics()
    table = tabulate(rows, headers=headers, tablefmt="presto")
    print(table)
    return rows


def biometric_logs():
    headers, rows = sql_biometric_logs(profile_id)

    table = tabulate(rows, headers=headers, tablefmt="presto")
    print(table)
    return rows


def biometric_add(bio_vals):
    print()
    # print("New biometric log: " + name + "\n")

    cur = con.cursor()
    bio_names = {x[0]: x for x in sql_biometrics()[1]}

    results = []
    for id, value in bio_vals.items():
        bio = bio_names[id]
        results.append({"id": id, "name": bio[1], "value": value, "unit": bio[2]})

    table = tabulate(results, headers="keys", tablefmt="presto")
    print(table)

    # TODO: print current profile and date?

    confirm = input("\nConfirm add biometric? [Y/n] ")

    if confirm.lower() == "y":
        sql_biometric_add(bio_vals)
        print("not implemented ;]")



#!/usr/bin/env python3
import csv
import os
import sqlite3

NT_DB_NAME = "nt.sqlite"


def build_ntsqlite():
    # cd into this script's directory
    SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
    os.chdir(SCRIPT_DIR)

    # print("Cleanup...")
    if os.path.isfile(NT_DB_NAME):
        os.remove(NT_DB_NAME)

    # print(f"\nPack {NT_DB_NAME}")
    con = sqlite3.connect(NT_DB_NAME)
    cur = con.cursor()

    # print("\n-> Create tables")
    with open("tables.sql") as tables:
        cur.executescript(tables.read())

    # print("-> Populate data")
    for p in os.listdir("data"):
        if not p.endswith(".csv"):
            continue
        t = os.path.splitext(os.path.basename(p))[0]
        p = os.path.join("data", p)

        # Loop over CSV files
        with open(p) as f:
            reader = csv.DictReader(f)
            q = ",".join("?" * len(reader.fieldnames))
            reader = csv.reader(f)
            cur.executemany(f"INSERT INTO {t} VALUES ({q});", reader)

    cur.close()
    con.commit()
    con.close()
    # print("\nDone!")
    return True


if __name__ == "__main__":
    build_ntsqlite()



# -*- coding: utf-8 -*-
"""
Created on Sat Mar 23 13:09:07 2019

@author: shane
"""

import os
import json

from .. import NUTRA_DIR

SCRIPT_DIR = os.path.abspath(os.path.dirname(__file__))

# TODO: init, handle when it doesn't exist yet
prefs_file = f"{NUTRA_DIR}/prefs.json"
if os.path.isfile(prefs_file):
    prefs = json.load(open(prefs_file))
else:
    prefs = dict()

REMOTE_HOST = "https://nutra-server.herokuapp.com"
SERVER_HOST = prefs.get("NUTRA_CLI_OVERRIDE_LOCAL_SERVER_HOST", REMOTE_HOST)

TESTING = int(os.getenv("NUTRA_CLI_NO_ARGS_INJECT_MOCKS", int(False)))
VERBOSITY = prefs.get("VERBOSITY", 1)


profile_id = prefs.get("current_user")  # guid retrieved by __init__ in .sqlfuncs
email = prefs.get("email")
login_token = prefs.get("token")



import os
import shutil
import sqlite3
import sys
import tarfile
import time
import urllib.request

from .... import __db_target_usda__, USDA_DB_NAME
from ... import NUTRA_DIR


# Onboarding function
def usda_init():
    # TODO: validate version against __db_target_usda__, return <True or False>
    # TODO: handle resource moved on Bitbucket or version mismatch due to manual overwrite?
    if USDA_DB_NAME not in os.listdir(NUTRA_DIR):
        """Downloads and unpacks the nt-sqlite3 db"""

        def reporthook(count, block_size, total_size):
            """Shows download progress"""
            global start_time
            if count == 0:
                start_time = time.time()
                time.sleep(0.01)
                return
            duration = time.time() - start_time
            progress_size = int(count * block_size)
            speed = int(progress_size / (1024 * duration))
            percent = int(count * block_size * 100 / total_size)
            sys.stdout.write(
                "\r...%d%%, %d MB, %d KB/s, %d seconds passed"
                % (percent, progress_size / (1024 * 1024), speed, duration)
            )
            sys.stdout.flush()

        # Download usda.sqlite.tar.xz
        url = f"https://bitbucket.org/dasheenster/nutra-utils/downloads/{USDA_DB_NAME}-{__db_target_usda__}.tar.xz"
        print(f"curl -L {url} -o {USDA_DB_NAME}.tar.xz")
        urllib.request.urlretrieve(
            url,
            f"{NUTRA_DIR}/{USDA_DB_NAME}.tar.xz",
            reporthook,
        )
        print()

        # Extract the archive
        # NOTE: in sql.__init__() we verify version == __db_target_usda__, and if needed invoke this method with force_install=True
        with tarfile.open(f"{NUTRA_DIR}/{USDA_DB_NAME}.tar.xz", mode="r:xz") as f:
            try:
                print(f"tar xvf {USDA_DB_NAME}.tar.xz")
                f.extractall(NUTRA_DIR)
            except Exception as e:
                print(repr(e))
                print("ERROR: corrupt tarball, removing. Please try init again")
                print("rm -rf ~/.nutra/usda")
                # shutil.rmtree(NUTRA_DIR)
                exit()
        print(f"==> done downloading {USDA_DB_NAME}")


# verify_usda(__db_target_usda__)

# Connect to DB
# TODO: support as customizable env var ?
db_path = os.path.join(NUTRA_DIR, USDA_DB_NAME)
if os.path.isfile(db_path):
    con = sqlite3.connect(db_path)
    # con.row_factory = sqlite3.Row  # see: https://chrisostrouchov.com/post/python_sqlite/
else:
    # print("warn: usda database doesn't exist, please run init")
    # print("info: init not implemented, manually copy")
    con = None


def _sql(query, headers=False):
    """Executes a SQL command to usda.sqlite"""
    # TODO: DEBUG flag or VERBOSITY level in prefs.json ... Print off all queries

    cur = con.cursor()
    result = cur.execute(query)
    rows = result.fetchall()
    if headers:
        headers = [x[0] for x in result.description]
        return headers, rows
    return rows



from ...persistence import profile_id
from .nt.funcs import nt_ver, sql_profile_guid_from_id
from .usda.funcs import usda_ver

# TODO: prompt to create profile if copying default `prefs.json` with profile_id: -1 (non-existent)
if profile_id is None:
    profile_guid = None
else:
    profile_guid = sql_profile_guid_from_id(profile_id)


# TODO: Verify version
__db_version_nt__ = nt_ver()
__db_version_usda__ = usda_ver()



# -*- coding: utf-8 -*-
"""
Created on Sat Oct 27 20:28:06 2018

@author: shane

This file is part of nutra, a nutrient analysis program.
    https://github.com/nutratech/cli
    https://pypi.org/project/nutra/

nutra is an extensible nutrient analysis and composition application.
Copyright (C) 2018-2020  Shane Jaroch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


import shutil

from fuzzywuzzy import fuzz
from tabulate import tabulate

from ..utils import (
    FOOD_NAME_TRUNC,
    NUTR_ID_KCAL,
    NUTR_IDS_AMINOS,
    NUTR_IDS_FLAVONES,
    SEARCH_LIMIT,
)
from ..persistence.sql.usda.funcs import (
    _sql,
    analyze_foods,
    nutrients_details,
    nutrients_overview,
    sort_foods,
    sort_foods_by_kcal,
)


def list_nutrients():

    headers, nutrients = nutrients_details()
    # TODO: include in SQL table cache?
    headers.append("avg_rda")
    nutrients = [list(x) for x in nutrients]
    for n in nutrients:
        rda = n[1]
        val = n[6]
        if rda:
            n.append(round(100 * val / rda, 1))
        else:
            n.append(None)

    table = tabulate(nutrients, headers=headers, tablefmt="simple")
    print(table)
    return nutrients


# -------------------------
# Sort
# -------------------------
def sort_foods_by_nutrient_id(id, by_kcal=False):
    # TODO: sub shrt_desc for long if available, and support FOOD_NAME_TRUNC
    results = sort_foods(id)
    results = [list(x) for x in results][:SEARCH_LIMIT]

    nutrients = nutrients_overview()
    nutrient = nutrients[id]
    unit = nutrient[2]

    headers = ["food", "fdgrp", f"val ({unit})", "kcal", "long_desc"]

    table = tabulate(results, headers=headers, tablefmt="simple")
    print(table)
    return results


def sort_foods_by_kcal_nutrient_id(id):
    results = sort_foods_by_kcal(id)
    results = [list(x) for x in results][:SEARCH_LIMIT]

    nutrients = nutrients_overview()
    nutrient = nutrients[id]
    unit = nutrient[2]

    headers = ["food", "fdgrp", f"val ({unit})", "kcal", "long_desc"]

    table = tabulate(results, headers=headers, tablefmt="simple")
    print(table)
    return results


# -------------------------
# Search
# -------------------------
def search_results(words):
    food_des = _sql("SELECT * FROM food_des;")

    query = " ".join(words)
    scores = {f[0]: fuzz.token_set_ratio(query, f[2]) for f in food_des}
    scores = sorted(scores.items(), key=lambda x: x[1], reverse=True)[:SEARCH_LIMIT]

    food_ids = [s[0] for s in scores]
    nut_data = analyze_foods(food_ids)

    # Tally foods
    foods_nutrients = {}
    for food_id, nutr_id, nutr_val in nut_data:
        if food_id not in foods_nutrients:
            foods_nutrients[food_id] = {nutr_id: nutr_val}  # init dict
        else:
            foods_nutrients[food_id][nutr_id] = nutr_val

    # TODO: include C/F/P macro ratios as column?
    results = []
    food_des = {f[0]: f for f in food_des}
    for score in scores:
        food_id = score[0]
        score = score[1]

        food = food_des[food_id]
        fdgrp_id = food[1]
        long_desc = food[2]
        shrt_desc = food[3]

        nutrients = foods_nutrients[food_id]
        result = {
            "food_id": food_id,
            "fdgrp_id": fdgrp_id,
            # TODO: get more details from another function, maybe enhance food_details() ?
            # "fdgrp_desc": cache.fdgrp[fdgrp_id]["fdgrp_desc"],
            # "data_src": cache.data_src[data_src_id]["name"],
            "long_desc": shrt_desc if shrt_desc else long_desc,
            "score": score,
            "nutrients": nutrients,
        }
        results.append(result)

    tabulate_search(results)


def tabulate_search(results):
    # Current terminal size
    # TODO: display "nonzero/total" report nutrients, aminos, and flavones.. sometimes zero values are not useful
    # TODO: macros, ANDI score, and other metrics on preview
    # bufferwidth = shutil.get_terminal_size()[0]
    bufferheight = shutil.get_terminal_size()[1]

    headers = [
        "food",
        "fdgrp",
        "kcal",
        "food_name",
        "Nutr",
        "Amino",
        "Flav",
    ]
    rows = []
    for i, r in enumerate(results):
        if i == bufferheight - 4:
            break
        food_id = r["food_id"]
        # TODO: dynamic buffer
        # food_name = r["long_desc"][:45]
        # food_name = r["long_desc"][:bufferwidth]
        food_name = r["long_desc"][:FOOD_NAME_TRUNC]
        # TODO: decide on food group description?
        # fdgrp_desc = r["fdgrp_desc"]
        fdgrp = r["fdgrp_id"]

        nutrients = r["nutrients"]
        kcal = nutrients.get(NUTR_ID_KCAL)
        len_aminos = len(
            [nutrients[n_id] for n_id in nutrients if int(n_id) in NUTR_IDS_AMINOS]
        )
        len_flavones = len(
            [nutrients[n_id] for n_id in nutrients if int(n_id) in NUTR_IDS_FLAVONES]
        )

        row = [
            food_id,
            fdgrp,
            kcal,
            food_name,
            len(nutrients),
            len_aminos,
            len_flavones,
        ]
        rows.append(row)
        # avail_buffer = bufferwidth - len(food_id) - 15
        # if len(food_name) > avail_buffer:
        #     rows.append([food_id, food_name[:avail_buffer] + "..."])
        # else:
        #     rows.append([food_id, food_name])
    table = tabulate(rows, headers=headers, tablefmt="simple")
    print(table)
    return rows



def print_nutprogbar(food_amts, food_analyses, nutrients):
    def tally(nut_percs):
        for nut in nut_percs:
            # TODO: get RDA values from nt DB, tree node nested organization
            print(nut)

    food_analyses = {
        x[0]: {y[1]: y[2] for y in food_analyses if y[0] == x[0]} for x in food_analyses
    }

    # print(food_ids)
    # print(food_analyses)

    nut_amts = {}

    for id, grams in food_amts.items():
        r = grams / 100.0
        analysis = food_analyses[id]
        for n, amt in analysis.items():
            if n not in nut_amts:
                nut_amts[n] = amt
            else:
                nut_amts[n] += amt

    nut_percs = {}

    for id, amt in nut_amts.items():
        # TODO: if not rda, show raw amounts?
        if type(nutrients[id][1]) == float:
            nut_percs[id] = round(amt / nutrients[id][1], 3)

    tally(nut_percs)
    return nut_percs



from . import con, _sql, __db_target_usda__


def usda_ver():
    """Gets version string for usda.sqlite database"""
    if con is None:
        return None
    query = "SELECT * FROM version;"
    result = _sql(query)
    return result[-1][1]


# Verify version
# try:
#     __db_version_usda__ = usda_ver()
#     if __db_target_usda__ != __db_version_usda__:
#         print(
#             f"NOTE: target db ({__db_target_usda__}) differs from current ({__db_version_usda__}).. downloading target"
#         )
#         verify_usda(__db_target_usda__, force_install=True)
#         print("NOTE: please run your command again now")
#         exit()
# except Exception as e:
#     print(repr(e))
#     print("ERROR: corrupt database.. downloading fresh")
#     verify_usda(__db_target_usda__, force_install=True)
#     print("NOTE: please run your command again now")
#     exit()


# ----------------------
# USDA  functions
# ----------------------


def fdgrp():
    query = "SELECT * FROM fdgrp;"
    result = _sql(query)
    return {x[0]: x for x in result}


def food_details(food_ids):
    """Readable human details for foods"""
    query = "SELECT * FROM food_des WHERE id in (%s)"
    food_ids = ",".join(str(x) for x in set(food_ids))
    return _sql(query % food_ids)


def nutrients_overview():
    query = "SELECT * FROM nutrients_overview;"
    result = _sql(query)
    return {x[0]: x for x in result}


def nutrients_details():
    query = "SELECT * FROM nutrients_overview;"
    return _sql(query, headers=True)


def servings(food_ids):
    """Food servings"""
    # TODO: apply connective logic from `sort_foods()` IS ('None') ?
    query = """
SELECT
  serv.food_id,
  serv.msre_id,
  serv_desc.msre_desc,
  serv.grams
FROM
  serving serv
  LEFT JOIN serv_desc ON serv.msre_id = serv_desc.id
WHERE
  serv.food_id IN (%s);
"""
    food_ids = ",".join(str(x) for x in set(food_ids))
    return _sql(query % food_ids)


def analyze_foods(food_ids):
    """Nutrient analysis for foods"""
    query = """
SELECT
  id,
  nutr_id,
  nutr_val
FROM
  food_des
  INNER JOIN nut_data ON food_des.id = nut_data.food_id
WHERE
  food_des.id IN (%s);
"""
    food_ids = ",".join(str(x) for x in set(food_ids))
    return _sql(query % food_ids)


def sort_foods(nutr_id, fdgrp_ids=None):
    """Sort foods by nutr_id per 100 g"""
    query = """
SELECT
  nut_data.food_id,
  fdgrp_id,
  nut_data.nutr_val,
  kcal.nutr_val AS kcal,
  long_desc
FROM
  nut_data
  INNER JOIN food_des food ON food.id = nut_data.food_id
  INNER JOIN nutr_def ndef ON ndef.id = nut_data.nutr_id
  INNER JOIN fdgrp ON fdgrp.id = fdgrp_id
  LEFT JOIN nut_data kcal ON food.id = kcal.food_id
    AND kcal.nutr_id = 208
WHERE
  nut_data.nutr_id = {0}"""
    if fdgrp_ids:
        query += """
  AND (fdgrp_id IN ({1}))"""
    query += """
ORDER BY
  nut_data.nutr_val DESC;"""
    if fdgrp_ids:
        fdgrp_ids = ",".join([str(x) for x in set(fdgrp_ids)])
        return _sql(query.format(nutr_id, fdgrp_ids))
    return _sql(query.format(nutr_id))


def sort_foods_by_kcal(nutr_id, fdgrp_ids=None):
    """Sort foods by nutr_id per 200 kcal"""
    query = """
SELECT
  nut_data.food_id,
  fdgrp_id,
  ROUND((nut_data.nutr_val * 200 / kcal.nutr_val), 2) AS nutr_val,
  kcal.nutr_val AS kcal,
  long_desc
FROM
  nut_data
  INNER JOIN food_des food ON food.id = nut_data.food_id
  INNER JOIN nutr_def ndef ON ndef.id = nut_data.nutr_id
  INNER JOIN fdgrp ON fdgrp.id = fdgrp_id
  -- filter out NULL kcal
  INNER JOIN nut_data kcal ON food.id = kcal.food_id
    AND kcal.nutr_id = 208
    AND kcal.nutr_val > 0
WHERE
  nut_data.nutr_id = {0}"""
    if fdgrp_ids:
        query += """
  AND (fdgrp_id IN ({1}))"""
    query += """
ORDER BY
  (nut_data.nutr_val / kcal.nutr_val) DESC;"""
    if fdgrp_ids:
        fdgrp_ids = ",".join([str(x) for x in set(fdgrp_ids)])
        return _sql(query.format(nutr_id, fdgrp_ids))
    return _sql(query.format(nutr_id))



import os
import sqlite3

from .... import NT_DB_NAME, NUTRA_DIR, __db_target_usda__

# Connect to DB
db_path = os.path.join(NUTRA_DIR, NT_DB_NAME)
if os.path.isfile(db_path):
    con = sqlite3.connect(db_path)
    con.row_factory = sqlite3.Row
else:
    # print("warn: nt database doesn't exist, please run init")
    # print("info: init not implemented, manually build db with ntsqlite README")
    con = None


def _sql(query, args=None, headers=False):
    """Executes a SQL command to nt.sqlite"""
    cur = con.cursor()

    # TODO: DEBUG flag in prefs.json ... Print off all queries
    if args:
        if type(args) == list:
            result = cur.executemany(query, args)
        else:  # tuple
            result = cur.execute(query, args)
    else:
        result = cur.execute(query)
    rows = result.fetchall()
    if headers:
        headers = [x[0] for x in result.description]
        return headers, rows
    return rows



# -*- coding: utf-8 -*-
"""
Created on Fri Jan 31 15:19:53 2020

@author: shane

This file is part of nutra, a nutrient analysis program.
    https://github.com/nutratech/cli
    https://pypi.org/project/nutra/

nutra is an extensible nutrient analysis and composition application.
Copyright (C) 2018-2020  Shane Jaroch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import pytest

from .. import __db_target_nt__, __db_target_usda__
from ..persistence.sql.nt import funcs as nt_funcs
from ..persistence.sql.usda import funcs as usda_funcs
from ..services import _init

# TODO: integration tests.. create user, recipe, log.. analyze & compare


def test_0_init():
    result = _init()
    assert result


def test_1_usda_sqlfuncs():
    version = usda_funcs.usda_ver()
    assert version == __db_target_usda__
    result = usda_funcs.nutrients_details()
    assert len(result[1]) == 186

    result = usda_funcs.servings([9050, 9052])
    assert len(result) == 3

    result = usda_funcs.analyze_foods([23567, 23293])
    assert len(result) == 188

    result = usda_funcs.sort_foods(789)
    assert len(result) == 415
    result = usda_funcs.sort_foods(789, [100])
    assert len(result) == 1

    result = usda_funcs.sort_foods_by_kcal(789)
    assert len(result) == 246
    result = usda_funcs.sort_foods_by_kcal(789, [1100])
    assert len(result) == 127


def test_2_nt_sqlfuncs():
    version = nt_funcs.nt_ver()
    assert version == __db_target_nt__



import os

from .. import NT_DB_NAME, NUTRA_DIR, ROOT_DIR
from ..ntsqlite.sql import build_ntsqlite
from ..persistence.sql.usda import usda_init

SCRIPT_DIR = os.path.abspath(os.path.dirname(__file__))


def _init():
    """
    TODO:   Check for:
        1. .nutra folder
        2. usda
        3a. nt
        3b. default profile?
        4. prefs.json
    """
    print("Nutra directory  ", end="")
    if not os.path.isdir(NUTRA_DIR):
        os.makedirs(NUTRA_DIR, 0o755)
    print(u"\u2713")

    # TODO: print off checks, return False if failed
    print("USDA db          ", end="")
    usda_init()
    print(u"\u2713")

    print("Nutra db         ", end="")
    build_ntsqlite()
    ntsqlite_buildpath = os.path.join(ROOT_DIR, "ntsqlite", "sql", NT_DB_NAME)
    # TODO: don't overwrite, verbose toggle for download, option to upgrade
    ntsqlite_destination = os.path.join(NUTRA_DIR, NT_DB_NAME)
    if os.path.isfile(ntsqlite_destination):
        print(u"\u2713")
        print(
            "WARN: upgrades not supported, please remove ntdb file or ignore this warning"
        )
    else:
        os.rename(ntsqlite_buildpath, ntsqlite_destination)
        print(u"\u2713")

    print("\nAll checks have passed!")
    return True



# -*- coding: utf-8 -*-
"""
Created on Fri Jan 31 16:02:19 2020

@author: shane

This file is part of nutra, a nutrient analysis program.
    https://github.com/nutratech/cli
    https://pypi.org/project/nutra/

nutra is an extensible nutrient analysis and composition application.
Copyright (C) 2018-2020  Shane Jaroch

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import argcomplete
import argparse
import sys
import traceback

from colorama import init as colorama_init

from . import (
    __db_target_nt__,
    __db_target_usda__,
    __title__,
    __version__,
)

from .parsers import (
    analyze,
    bio,
    bio_log,
    bio_log_add,
    day,
    init,
    nutrients,
    recipe,
    recipe_add,
    recipe_edit,
    search,
    sort,
    sync,
    sync_login,
    sync_register,
)
from .persistence import TESTING, VERBOSITY

colorama_init()

# TODO:
# self.init function and state flow
# - display full food name in results
# - display refuse
# - function to list out nutrients and info on them
# - sort function
# - nested nutrient tree, like: http://www.whfoods.com/genpage.php?tname=nutrientprofile&dbid=132


def build_argparser():

    arg_parser = argparse.ArgumentParser(prog=__title__)
    arg_parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"{__title__} cli version {__version__}  [DB usda v{__db_target_usda__}, nt v{__db_target_nt__}]",
    )

    # --------------------------
    # Sub-command parsers
    # --------------------------
    subparsers = arg_parser.add_subparsers(title="nutra subcommands")

    # --------------------------
    # Init subcommand
    # --------------------------
    init_parser = subparsers.add_parser(
        "init", help="setup profiles, USDA and NT database"
    )
    init_parser.set_defaults(func=init)

    # --------------------------
    # Search subcommand
    # --------------------------
    search_parser = subparsers.add_parser(
        "search", help="search foods by name, list overview info"
    )
    search_parser.add_argument(
        "terms",
        type=str,
        nargs="+",
        help='search query, e.g. "grass fed beef" or "ultraviolet mushrooms"',
    )
    search_parser.set_defaults(func=search)

    # --------------------------
    # Sort subcommand
    # --------------------------
    sort_parser = subparsers.add_parser("sort", help="sort foods by nutrient ID")
    sort_parser.add_argument(
        "--kcal",
        "-c",
        action="store_true",
        help="sort by value per 200 kcal, instead of per 100 g",
    )
    sort_parser.add_argument("nutr_id", type=int)
    sort_parser.set_defaults(func=sort)

    # --------------------------
    # Analyze subcommand
    # --------------------------
    analyze_parser = subparsers.add_parser("anl", help="analyze food(s)")
    analyze_parser.add_argument("food_id", type=int, nargs="+")
    analyze_parser.add_argument(
        "-g",
        dest="grams",
        type=float,
        help="analyze for custom number of grams (default is 100g)",
    )
    analyze_parser.set_defaults(func=analyze)

    # --------------------------
    # Day (analyze-day) subcommand
    # --------------------------
    day_parser = subparsers.add_parser(
        "day", help="analyze a DAY.csv file, RDAs optional"
    )
    day_parser.add_argument(
        "food_log", type=str, nargs="+", help="path to CSV file of food log"
    )
    day_parser.add_argument(
        "--rda",
        "-r",
        help="provide a custom RDA file in csv format",
    )
    day_parser.set_defaults(func=day)

    # --------------------------
    # Recipe subcommand
    # --------------------------
    recipe_parser = subparsers.add_parser("recipe", help="list and analyze recipes")

    recipe_subparsers = recipe_parser.add_subparsers(title="recipe subcommands")

    recipe_add_parser = recipe_subparsers.add_parser("add", help="add a recipe")
    recipe_add_parser.add_argument("name")
    recipe_add_parser.add_argument(
        "food_amt", nargs="+", help="food_id,grams e.g. 1001,15 for 15 grams butter"
    )
    recipe_add_parser.set_defaults(func=recipe_add)

    recipe_edit_parser = recipe_subparsers.add_parser("edit", help="edit a recipe")
    recipe_edit_parser.add_argument("recipe_id", type=int, help="edit recipe by ID")
    recipe_edit_parser.set_defaults(func=recipe_edit)

    # default case
    recipe_parser.add_argument(
        "-n", dest="recipe_id", type=int, help="analyze recipe by ID"
    )
    recipe_parser.set_defaults(func=recipe)

    # --------------------------
    # Biometric subcommand
    # --------------------------
    bio_parser = subparsers.add_parser("bio", help="view, add, remove biometric logs")
    bio_subparsers = bio_parser.add_subparsers(title="biometric subcommands")

    # Log
    bio_log_parser = bio_subparsers.add_parser("log", help="manage biometric logs")
    bio_log_subparsers = bio_log_parser.add_subparsers(
        title="biometric log subcommands"
    )
    bio_log_parser.set_defaults(func=bio_log)

    bio_log_add_parser = bio_log_subparsers.add_parser(
        "add", help="add a biometric log"
    )
    bio_log_add_parser.add_argument(
        "biometric_val", help="id,value pairs, e.g. 22,59 23,110 24,65 ", nargs="+"
    )
    bio_log_add_parser.set_defaults(func=bio_log_add)

    # default case
    bio_parser.set_defaults(func=bio)

    # --------------------------
    # Nutrient subcommand
    # --------------------------
    nutrient_parser = subparsers.add_parser(
        "nt", help="list out nutrients and their info"
    )
    nutrient_parser.set_defaults(func=nutrients)

    # --------------------------
    # Sync subparsers
    # --------------------------
    sync_parser = subparsers.add_parser(
        "sync", help="sync, login and account management"
    )

    sync_subparsers = sync_parser.add_subparsers(title="sync subcommands")

    # Login
    sync_login_parser = sync_subparsers.add_parser(
        "login", help="login and store token locally"
    )
    sync_login_parser.add_argument("email")
    sync_login_parser.set_defaults(func=sync_login)

    # Register
    sync_register_parser = sync_subparsers.add_parser(
        "register", help="register an account to set up sync"
    )
    sync_register_parser.set_defaults(func=sync_register)

    # default case
    sync_parser.set_defaults(func=sync)

    return arg_parser


def main(argv=None):
    if argv is None:
        argv = sys.argv
    arg_parser = build_argparser()
    # Used for testing
    if TESTING and len(sys.argv) < 2:
        # --------------------------------
        # sys.argv = [
        #     "./nutra",
        #     "day",
        #     "~/.nutra/rocky.csv",
        #     # "~/.nutra/rocky-mom.csv",
        #     "-r",
        #     "~/.nutra/dog-rdas-18lbs.csv",
        # ]
        # --------------------------------
        # sys.argv = [
        #     "./nutra",
        #     "day",
        #     "~/.nutra/rocky.csv",
        # ]
        # --------------------------------
        # sys.argv = [
        #     "./nutra",
        #     "day",
        #     "resources/dog-simple.csv",
        #     "-r",
        #     "resources/dog-rdas-18lbs.csv",
        # ]
        # --------------------------------
        # sys.argv = ["./nutra"]
        # sys.argv = ["./nutra" "-h"]
        # sys.argv = ["./nutra" "-v"]
        sys.argv = ["./nutra", "init"]
        # sys.argv = ["./nutra", "sort"]
        # sys.argv = ["./nutra", "sort", "789"]
        # sys.argv = ["./nutra", "sort", "-c", "789"]
        # sys.argv = ["./nutra", "anl", "9050", "9052"]
        # sys.argv = ["./nutra", "anl", "-g", "85", "23294"]
        # sys.argv = ["./nutra", "recipe"]
        # sys.argv = ["./nutra", "recipe", "-n", "1"]
        # sys.argv = ["./nutra", "recipe", "add", "Test", "1001,15"]
        # sys.argv = ["./nutra", "bio"]
        # sys.argv = ["./nutra", "bio", "log", "add", "22,59", "23,110", "24,65"]
        # sys.argv = ["./nutra", "nt"]
        # sys.argv = ["./nutra", "search", "grass", "fed", "beef"]
        # sys.argv = ["./nutra", "search", "grass"]
        # sys.argv = ["./nutra", "sync"]
        # sys.argv = ["./nutra", "sync", "login"]
    try:
        argcomplete.autocomplete(arg_parser)
        args = arg_parser.parse_args()
        # args, unknown = arg_parser.parse_known_args()
        if hasattr(args, "func"):
            subparsers_actions = [
                action
                for action in arg_parser._actions
                if isinstance(action, argparse._SubParsersAction)
            ]
            subparsers = subparsers_actions[0].choices

            # Run the cmd function
            args.func(args, arg_parser=arg_parser, subparsers=subparsers)
        else:
            arg_parser.print_help()
    except Exception as e:
        print("There was an unforseen error: ", repr(e))
        if TESTING or VERBOSITY > 0:
            trace = "\n".join(traceback.format_tb(e.__traceback__))
            print(trace)
        # arg_parser.print_help()



# -*- coding: utf-8 -*-
"""
Created on Sat Oct 13 16:30:30 2018

@author: shane
"""

import glob
import os
import shutil
import subprocess
import sys
import traceback

from setuptools import setup, find_packages

from ntclient import __version__

# Old pip doesn't respect `python_requires'
if sys.version_info < (3, 6, 5):
    ver = ".".join([str(x) for x in sys.version_info[0:3]])
    print("ERROR: nutra requires Python 3.6.5 or later to install")
    print("HINT:  You're running Python " + ver)
    exit(1)

# cd to parent dir of setup.py
os.chdir(os.path.dirname(os.path.abspath(__file__)))
shutil.rmtree("dist", True)

CLASSIFIERS = [
    "Environment :: Console",
    "Intended Audience :: End Users/Desktop",
    "Intended Audience :: Science/Research",
    "Intended Audience :: Healthcare Industry",
    "Intended Audience :: Education",
    "Development Status :: 3 - Alpha",
    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
]

REQUIREMENTS = [
    "colorama",
    "fuzzywuzzy",
    "pytest",
    "python-dotenv",
    "python-Levenshtein",
    "requests",
    "tabulate",
]

README = open("README.rst").read()

PKG_NAME = "nutra"

setup(
    name=PKG_NAME,
    author="gamesguru",
    author_email="mathmuncher11@gmail.com",
    classifiers=CLASSIFIERS,
    install_requires=REQUIREMENTS,
    python_requires=">=3.6.5",
    packages=find_packages(exclude=["test"]),
    include_package_data=True,
    scripts=glob.glob("scripts/*"),
    # entry_points={"console_scripts": ["nutra=ntclient.__main__:main"]},
    description="Home and office nutrient tracking software",
    long_description=README,
    long_description_content_type="text/x-rst",
    url="https://github.com/nutratech/cli",
    license="GPL v3",
    version=__version__,
)

# Clean up
shutil.rmtree(f"{PKG_NAME}.egg-info", True)
shutil.rmtree("__pycache__", True)
shutil.rmtree(".pytest_cache", True)



import os

from .services import _init
from .services.analyze import day_analyze, foods_analyze
from .services.recipe import (
    recipe_add as _recipe_add,
    recipe_edit as _recipe_edit,
    recipe_overview,
    recipes_overview,
)
from .services.biometrics import (
    biometrics,
    biometric_add,
    biometric_logs,
)
from .services.usda import (
    list_nutrients,
    search_results,
    sort_foods_by_kcal_nutrient_id,
    sort_foods_by_nutrient_id,
)

# TODO: rethink arg_parse=None argument, and hasattr() approach (bottom of __main__.py)


def init(args, arg_parser=None, **kwargs):
    _init()


# --------------------------
# Nutrients, search and sort
# --------------------------
def nutrients(args, arg_parser=None, **kwargs):
    return list_nutrients()


def search(args, arg_parser=None, subparsers=None):
    """ Searches all dbs, foods, recipes, recents and favorites. """
    if args.terms:
        return search_results(words=args.terms)
    else:
        subparsers["search"].print_help()


def sort(args, arg_parser=None, subparsers=None):
    nutr_id = args.nutr_id
    if not nutr_id:
        subparsers["sort"].print_help()
    elif args.kcal:
        return sort_foods_by_kcal_nutrient_id(nutr_id)
    else:
        return sort_foods_by_nutrient_id(nutr_id)


# --------------------------
# Analysis and Day scoring
# --------------------------
def analyze(args, arg_parser=None, subparsers=None):
    food_ids = args.food_id
    grams = args.grams

    if not food_ids:
        subparsers["anl"].print_help()
    else:
        return foods_analyze(food_ids, grams)


def day(args, arg_parser=None, subparsers=None):
    day_csv_paths = args.food_log
    day_csv_paths = [os.path.expanduser(x) for x in day_csv_paths]
    if args.rda:
        rda_csv_path = os.path.expanduser(args.rda)

    if not day_csv_paths:
        subparsers["day"].print_help()
    elif not args.rda:
        return day_analyze(day_csv_paths)
    else:
        return day_analyze(day_csv_paths, rda_csv_path=rda_csv_path)


# --------------------------
# Biometrics
# --------------------------
def bio(args, arg_parser=None, subparsers=None):
    return biometrics()


def bio_log(args, arg_parser=None, subparsers=None):
    return biometric_logs()


def bio_log_add(args, arg_parser=None, subparsers=None):
    bio_vals = {
        int(x.split(",")[0]): float(x.split(",")[1]) for x in args.biometric_val
    }

    return biometric_add(bio_vals)


# --------------------------
# Recipes
# --------------------------
def recipe(args, arg_parser=None, subparsers=None):
    if args.recipe_id:
        return recipe_overview(args.recipe_id)
    else:
        return recipes_overview()


def recipe_add(args, arg_parser=None, subparsers=None):
    food_amts = {int(x.split(",")[0]): float(x.split(",")[1]) for x in args.food_amt}
    return _recipe_add(args.name, food_amts)


def recipe_edit(args, arg_parser=None, subparsers=None):
    return _recipe_edit(args.recipe_id)


# --------------------------
# Sync
# --------------------------
def sync(args, arg_parser=None, subparsers=None):
    from .services.sync import sync as _sync

    _sync()


def sync_register(args, arg_parser=None, subparsers=None):
    from getpass import getpass
    from .services.sync import register

    print("not implemented ;]")
    return

    email = input("email: ")
    confirm_email = input("confirm email: ")
    password = getpass("password: ")
    confirm_password = getpass("confirm password: ")

    if email != confirm_email or password != confirm_password:
        print("Try again, email and password must match")
        return

    register(email, password)


def sync_login(args, arg_parser=None, subparsers=None):
    from getpass import getpass
    from .services.sync import login

    password = getpass("password: ")
    login(args.email, password)
